@@ -10,8 +10,6 @@
 
 #include "access/gist.h"
 #include "access/rtree.h"
-#include "utils/elog.h"
-#include "utils/palloc.h"
 #include "utils/builtins.h"
 
 #include "segdata.h"
@@ -25,8 +23,10 @@
 #define GIST_QUERY_DEBUG
 */
 
-extern void set_parse_buffer(char *str);
 extern int	seg_yyparse();
+extern void seg_yyerror(const char *message);
+extern void seg_scanner_init(const char *str);
+extern void seg_scanner_finish(void);
 
 /*
 extern int	 seg_yydebug;
@@ -101,16 +101,13 @@ seg_in(char *str)
 {
 	SEG		   *result = palloc(sizeof(SEG));
 
-	set_parse_buffer(str);
+	seg_scanner_init(str);
 
-	/*
-	 * seg_yydebug = 1;
-	 */
 	if (seg_yyparse(result) != 0)
-	{
-		pfree(result);
-		return NULL;
-	}
+		seg_yyerror("bogus input");
+
+	seg_scanner_finish();
+
 	return (result);
 }
 
@@ -798,7 +795,7 @@ seg_cmp(SEG * a, SEG * b)
 		if (b->l_ext == '~')
 			return 1;
 		/* can't get here unless data is corrupt */
-		elog(ERROR, "seg_cmp: bogus lower boundary types %d %d",
+		elog(ERROR, "bogus lower boundary types %d %d",
 			 (int) a->l_ext, (int) b->l_ext);
 	}
 
@@ -857,7 +854,7 @@ seg_cmp(SEG * a, SEG * b)
 		if (b->u_ext == '~')
 			return -1;
 		/* can't get here unless data is corrupt */
-		elog(ERROR, "seg_cmp: bogus upper boundary types %d %d",
+		elog(ERROR, "bogus upper boundary types %d %d",
 			 (int) a->u_ext, (int) b->u_ext);
 	}
 
@@ -882,7 +879,6 @@ seg_gt(SEG * a, SEG * b)
 	return seg_cmp(a, b) > 0;
 }
 
-
 bool
 seg_ge(SEG * a, SEG * b)
 {
