@@ -8,7 +8,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/optimizer/util/clauses.c,v 1.154.2.1 2003/12/09 01:56:41 tgl Exp $
+ *	  $Header: /cvsroot/pgsql/src/backend/optimizer/util/clauses.c,v 1.154.2.4 2005/04/14 21:44:35 tgl Exp $
  *
  * HISTORY
  *	  AUTHOR			DATE			MAJOR EVENT
@@ -1553,6 +1553,10 @@ eval_const_expressions_mutator(Node *node, List *active_fns)
 			FastAppend(&newargs, e);
 		}
 
+		/* If all the arguments were constant null, the result is just null */
+		if (FastListValue(&newargs) == NIL)
+			return (Node *) makeNullConst(coalesceexpr->coalescetype);
+
 		newcoalesce = makeNode(CoalesceExpr);
 		newcoalesce->coalescetype = coalesceexpr->coalescetype;
 		newcoalesce->args = FastListValue(&newargs);
@@ -1655,6 +1659,7 @@ evaluate_function(Oid funcid, Oid result_type, List *args,
 	bool		has_null_input = false;
 	List	   *arg;
 	FuncExpr   *newexpr;
+	char		result_typtype;
 
 	/*
 	 * Can't simplify if it returns a set.
@@ -1662,6 +1667,13 @@ evaluate_function(Oid funcid, Oid result_type, List *args,
 	if (funcform->proretset)
 		return NULL;
 
+	/*
+	 * Can't simplify if it returns RECORD, since it will be needing an
+	 * expected tupdesc which we can't supply here.
+	 */
+	if (funcform->prorettype == RECORDOID)
+		return NULL;
+
 	/*
 	 * Check for constant inputs and especially constant-NULL inputs.
 	 */
@@ -1691,6 +1703,15 @@ evaluate_function(Oid funcid, Oid result_type, List *args,
 		has_nonconst_input)
 		return NULL;
 
+	/*
+	 * Can't simplify functions returning composite types (mainly because
+	 * datumCopy() doesn't cope; FIXME someday when we have a saner
+	 * representation for whole-tuple results).
+	 */
+	result_typtype = get_typtype(funcform->prorettype);
+	if (result_typtype == 'c')
+		return NULL;
+
 	/*
 	 * OK, looks like we can simplify this operator/function.
 	 *
